"""
Copyright 2014-2015 Arve Kylling

This file is part of IRoPLUM.

IRoPLUM is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

IRoPLUM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with IRoPLUM.  If not, see <http://www.gnu.org/licenses/>.

"""
import numpy as np
import scipy.ndimage
import matplotlib.pyplot as plt
from scipy import interpolate
from  optparse import OptionParser

parser = OptionParser(usage="""Compare target (from SyntheticLineIntegrals.py) and
                             reconstructed images (from Reconstruction2D.py).""")
parser.add_option('--RRImageFile', dest='RRImageFile', default=None,
                  help="""Name of file holding Reconstructed Image.""") 
parser.add_option('--TargetImageFile', dest='TargetImageFile', default=None,
                  help="""Name of Targe Image File.""") 
(args,rest)   =  parser.parse_args()

fnTarget = args.TargetImageFile #'tmpTarget.npz'
fnRR     = args.RRImageFile     #'tmpRRImage.npz'

TargetFile = np.load(fnTarget)
Target = TargetFile.Image
Targetx = TargetFile.x
Targetz = TargetFile.z
print "Target", Target.min(), Target.max(), Target.shape, Targetz.shape, Targetz.shape
vmin = Target.min()
vmax = Target.max()
ticks = np.arange(vmin,vmax,0.2)

RRImageFile = np.load(fnRR)
RRImage = RRImageFile.Image
#print RRImageFile.files
RRx = RRImageFile.x
RRz = RRImageFile.z
RRxmid = RRImageFile.x_mid
RRzmid = RRImageFile.z_mid
print "RRImage", RRImage.min(), RRImage.max(), RRImage.shape
print RRx.shape, RRx
print RRz.shape, RRz
z, x = np.mgrid[slice(RRImageFile.z_mid.max(), RRImageFile.z_mid.min()-RRImageFile.dz/2 , -RRImageFile.dz),
                slice(RRImageFile.x_mid.min(), RRImageFile.x_mid.max()+RRImageFile.dx/2 , RRImageFile.dx)]

#f = interpolate.interp2d(z, x, RRImage, kind='cubic')
#print RRz

#print RRzmid.shape, RRxmid.shape, RRImage.shape
#tck = interpolate.bisplrep(z, x, RRImage, s=0)
#exit()
#znew = interpolate.bisplev(xnew[:,0], ynew[0,:], tck)

#f = interpolate.RectBivariateSpline(RRz, RRx, RRImage)

#z, x = np.mgrid[slice(TargetFile.z_mid.max(), TargetFile.z_mid.min()-TargetFile.dz/2 , -TargetFile.dz),
#                slice(TargetFile.x_mid.min(), TargetFile.x_mid.max()+TargetFile.dx/2 , TargetFile.dx)]

#RRImage = interpolate.bisplev(Targetz,Targetx, tck)
#RRImage = f(Targetz,Targetx)

print RRImage.shape, Target.shape
print "RRImage", RRImage.min(), RRImage.max(), RRImage.shape
#print RRx
#print RRz
#exit()
image = RRImage-Target #Target#  RRImage #

#fig, (ax1,ax2,ax3) = plt.subplots(1, 3, figsize=(12, 4))
fig = plt.subplots(1, 3, figsize=(18, 5))
plt.subplots_adjust(bottom=0.13,left=0.07,right=0.95)

ax1=plt.subplot(1,3,1)
im1=ax1.pcolor(Targetx, Targetz, Target, cmap='gist_ncar_r', vmin=vmin, vmax=vmax)
ax1.set_title("Target")
ax1.set_xlabel("Distance (m)")
ax1.set_ylabel("Heigth (m)")
margin=-100
ax1.axis([RRx.min()-margin, RRx.max()+margin, RRz.min()-margin, RRz.max()+margin])

cb = plt.colorbar(im1)#, ticks=ticks)
cblabel="Density (arb. units)"
cb.set_label(cblabel)#, size=cblabelsize)
cb.set_ticks(ticks)
cb.set_ticklabels(ticks)

ax2=plt.subplot(1,3,2)
#im2=ax2.pcolor(Targetx, Targetz, RRImage, cmap='Reds', vmin=vmin, vmax=vmax)
im2=ax2.pcolor(RRx, RRz, RRImage, cmap='gist_ncar_r', vmin=vmin, vmax=vmax)
ax2.set_title("Reconstruction")
ax2.set_xlabel("Distance (m)")
ax2.set_ylabel("Heigth (m)")
ax2.axis([RRx.min()-margin, RRx.max()+margin, RRz.min()-margin, RRz.max()+margin])
cb = plt.colorbar(im2)
cblabel="Density (arb. units)"
cb.set_label(cblabel)#, size=cblabelsize)
cb.set_ticks(ticks)
cb.set_ticklabels(ticks)

ax3=plt.subplot(1,3,3)
z_min=-0.3 #15
z_max= 0.3 #15
im3=ax3.pcolor(Targetx, Targetz, RRImage-Target, cmap='RdBu_r', vmin=z_min, vmax=z_max)
cb = plt.colorbar(im3)
ax3.set_title("Reconstruction-Target")
ax3.set_xlabel("Distance (m)")
ax3.set_ylabel("Heigth (m)")
ax3.axis([RRx.min()-margin, RRx.max()+margin, RRz.min()-margin, RRz.max()+margin])
cblabel="Density difference (arb. units)"
cb.set_label(cblabel)#, size=cblabelsize)

plt.show()
exit()
